var ogma = new Ogma({
  container: 'graph-container'
});

// always show labels
ogma.styles.addNodeRule({
  text: {
    minVisibleSize: 0
  },
  icon: function(node) {
    if (node.isVirtual()) {
      return {
        content: '\uf0e8',
        font: 'FontAwesome',
        color: 'white'
      };
    }
    return {
      content: '\uf233',
      font: 'FontAwesome',
      color: 'white'
    };
  }
});

function formatContent(ids) {
  if (!Array.isArray(ids)) {
    return ''; // this is for regular nodes
  }
  return (
    '<br/><br/>Contains:<ul/><li>"' + ids.join('"</li><li/>"') + '</li></ul>'
  );
}

// Show some info from the hovered node
ogma.tools.tooltip.onNodeHover(
  function(node) {
    // pick the color of the node
    var color = node.getAttribute('color');
    // Now change the title based on the type of node (grouped or not?)
    var title = node.isVirtual()
      ? 'Grouped Area&nbsp;'
      : 'ID: "' + node.getId() + '"&nbsp;';
    // and pick the content of the node if grouped;
    var content =
      node.isVirtual() && !isVisualGroup(node)
        ? node.getData('ids')
        : node.getId();
    return (
      '<div class="arrow"></div>' +
      '<div class="ogma-tooltip-header">' +
      '<span class="title">' +
      title +
      '<span class="line-color" style="background: ' +
      color +
      '">' +
      '&nbsp;' +
      '</span>' +
      formatContent(content) +
      '</span>' +
      '</div>'
    );
  },
  {
    className: 'ogma-tooltip' // tooltip container class to bind to css
  }
);

var groups;
var nodeGroupings;

ogma.parse
  .jsonFromUrl('network.json')
  .then(function(graph) {
    ogma.setGraph(graph);

    // eslint-disable-next-line es5/no-es6-static-methods
    groups = Array.from(new Set(ogma.getNodes().getData('area')));

    // transformation objects: create them now and use them later
    nodeGroupings = groups.map(function(group) {
      return createGroupingTransformation(group);
    });

    return ogma.layouts.force();
  })
  .then(function() {
    return ogma.view.locateGraph({ padding: 120 });
  });

// this is the duration of the animations
var transitionTime = 500;
// Menu at the top right
var form = document.querySelector('#ui');
form.addEventListener('change', updateUI);

document.querySelector('#layout').addEventListener('click', function(evt) {
  evt.preventDefault();
  ogma.layouts.force().then(function() {
    // if there are visual groups layout them too
    var opens = getOpens('area');
    if (
      Object.keys(opens).some(function(k) {
        return opens[k];
      })
    ) {
      var subNodesList = ogma
        .getNodes()
        .filter(function(n) {
          return isVisualGroup(n);
        })
        .getSubNodes();
      return Promise.all([
        subNodesList.map(function(nodes) {
          return ogma.layouts.force({ nodes: nodes });
        })
      ]);
    }
  });
});

// add the double click feature to open/close groups
ogma.events.onDoubleClick(function(evt) {
  if (evt.target) {
    if (evt.target.isNode && evt.target.isVirtual()) {
      // check if it's closed or open group
      var isOpen = isVisualGroup(evt.target);
      // update the UI accordingly now
      form['area'].forEach(function(checkbox) {
        if (checkbox.value === evt.target.getData('area')) {
          checkbox.checked = !isOpen;
        }
      });
      updateUI();
    }
    return;
  }
  ogma.view.zoomIn({ duration: 300 });
});

function isVisualGroup(node) {
  return (
    node.isVirtual() &&
    node
      .getSubNodes()
      .isVisible()
      .some(function(b) {
        return b;
      })
  );
}

function hexToRgb(hex) {
  var cleanHex = hex.slice(1);
  var bigint = parseInt(cleanHex, 16);
  var r = (bigint >> 16) & 255;
  var g = (bigint >> 8) & 255;
  var b = bigint & 255;

  return r + ',' + g + ',' + b;
}

function createGroupingTransformation(area) {
  var transformation = ogma.transformations.addNodeGrouping({
    selector: function(node) {
      return node.getData('area') === area;
    },
    groupIdFunction: function(node) {
      return node.getData('area');
    },
    nodeGenerator: function(nodeList, groupId) {
      var opens = getOpens('area');
      var originalColor = nodeList.getAttribute('color')[0];
      // make visual group color a bit lighter playing with the alpha channel
      var color = opens[nodeList.getData('area')[0]]
        ? 'rgba(' + hexToRgb(originalColor) + ', 0.4)'
        : originalColor;
      var radius = opens[nodeList.getData('area')[0]] ? 0 : nodeList.size * 8;
      return {
        id: 'grouped-node-' + groupId,
        attributes: {
          text: groupId + ' Group',
          radius: radius,
          color: color
        },
        // add some content to be picked up later on in the tooltip
        data: {
          ids: nodeList.getId(),
          area: groupId
        }
      };
    },
    showContents: function(node) {
      var opens = getOpens('area');
      return opens[node.getData('area')];
    },
    enabled: false // create the transformation but do not activate it
  });
  // add a prop on the transformation to know the criteria used here
  transformation.area = area;
  return transformation;
}

var edgeGrouping = ogma.transformations.addEdgeGrouping({
  enabled: false // create the transformation but do not activate it
});
var filterSwitch = ogma.transformations.addNodeFilter({
  criteria: function(node) {
    return node.isVirtual() || /^router/.test(node.getId()); // just keep the router nodes
  },
  enabled: false // create the transformation but do not activate it
});

// Disable the connectivity form when nodes are grouped
function updateConnectivityUI(enable) {
  for (var i = 0; i < 2; i++) {
    form.elements['edge-group'][i].disabled = !enable;
    form.elements['filter-switch'][i].disabled = !enable;
  }
}

function updateUI() {
  // Check what it should
  var nodeGrouped = getMode('node-group') === 'group';
  var edgeGrouped = getMode('edge-group') === 'group';
  var filterActive = getMode('filter-switch') === 'filter';
  var opens = getOpens('area');

  // instead of working out how to go from state A to state B
  // execute all transformation and for each one workout what to do
  return execute(filterSwitch, filterActive)
    .then(function() {
      return execute(edgeGrouping, edgeGrouped);
    })
    .then(function() {
      return executeGrouping(nodeGrouped, opens);
    })
    .then(function() {
      var someVisualGrouping = Object.keys(opens).some(function(area) {
        return opens[area];
      });
      return updateConnectivityUI(!nodeGrouped || someVisualGrouping);
    })
    .then(function() {
      return ogma.layouts.force();
    });
}

function executeGrouping(toGroup, opens) {
  return Promise.all(
    nodeGroupings.map(function(nodeGrouping) {
      // enable the transformation when either it has to node group or to visual group
      var enableTransformation =
        (toGroup && !opens[nodeGrouping.area]) || opens[nodeGrouping.area];
      if (nodeGrouping.isEnabled() === enableTransformation) {
        return nodeGrouping.refresh();
      }
      if (enableTransformation) {
        return nodeGrouping.enable(transitionTime);
      }
      return nodeGrouping.disable(transitionTime);
    })
  );
}

function execute(transformation, enable) {
  // avoid extra work if we're already in the state requested
  if (transformation.isEnabled() === enable) {
    return Promise.resolve();
  }
  // if now toggle the state with animation
  if (enable) {
    return transformation.enable(transitionTime);
  }
  return transformation.disable(transitionTime);
}

function getOpens(name) {
  var checkboxes = form[name];
  var opens = {};
  checkboxes.forEach(function(input) {
    opens[input.value] = input.checked;
  });
  return opens;
}

function getMode(id) {
  var select = form[id];
  var currentMode = Array.prototype.filter.call(select, function(input) {
    return input.checked;
  })[0].value; // IE inconsistency
  return currentMode;
}
